<?php

namespace Drupal\entity_browser_table\Plugin\Field\FieldWidget;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_browser\Plugin\Field\FieldWidget\EntityReferenceBrowserWidget;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;
use Drupal\node\Entity\NodeType;
use Drupal\taxonomy\Entity\Term;
use Drupal\Component\Utility\Html;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'entity_reference_browser_table_widget' widget.
 *
 * @FieldWidget(
 *   id = "entity_reference_browser_table_widget",
 *   label = @Translation("Entity Browser - Table"),
 *   multiple_values = TRUE,
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceBrowserTableWidget extends EntityReferenceBrowserWidget {

  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $id = Html::getUniqueId('field-' . $this->fieldDefinition->getName() . '-display-settings-wrapper');
    $element['field_widget_display'] = [
      '#title' => $this->t('Entity display plugin'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('field_widget_display'),
      '#options' => ['label' => t('Entity Label')],
      '#ajax' => [
        'callback' => [$this, 'updateSettingsAjax'],
        'wrapper' => $id,
      ],
    ];

    return $element;
  }

  /**
   * Builds the render array for displaying the current results as a table.
   *
   * @param string $details_id
   *   The ID for the details element.
   * @param string[] $field_parents
   *   Field parents.
   * @param \Drupal\Core\Entity\ContentEntityInterface[] $entities
   *   Array of referenced entities.
   *
   * @return array
   *   The render array for the current selection.
   */
  protected function displayCurrentSelection($details_id, $field_parents, $entities) {

    try {
      $field_widget_display = $this->fieldDisplayManager->createInstance(
        $this->getSetting('field_widget_display'),
        $this->getSetting('field_widget_display_settings') + [
          'entity_type' => $this->fieldDefinition->getFieldStorageDefinition()
            ->getSetting('target_type'),
        ]
      );

      $classes = ['entities-list'];
      if ($this->fieldDefinition->getFieldStorageDefinition()
          ->getCardinality() != 1) {
        $classes[] = 'sortable';
      }

      // The "Replace" button will only be shown if this setting is enabled in the
      // widget, and there is only one entity in the current selection.
      $replace_button_access = $this->getSetting('field_widget_replace') && (count($entities) === 1);


      $table = [
        '#type'       => 'table',
        '#header'     => ['Title', 'Type', 'Actions'],
        '#attributes' => ['class' => $classes],
      ];

      foreach ($entities as $row_id => $entity) {
        $display = $field_widget_display->view($entity);
        $edit_button_access = $this->getSetting('field_widget_edit') && $entity->access('update', $this->currentUser);
        if ($entity->getEntityTypeId() == 'file') {
          // On file entities, the "edit" button shouldn't be visible unless
          // the module "file_entity" is present, which will allow them to be
          // edited on their own form.
          $edit_button_access &= $this->moduleHandler->moduleExists('file_entity');
        }
        if (is_string($display)) {
          $display = ['#markup' => $display];
        }

        $entityType = $entity->getEntityType()->id();

        switch ($entityType) {
          case 'node':
            $type = ($entityType = NodeType::load($entity->bundle()))
              ? $entityType->label()
              : '';
            break;

          case 'media':
            $type = ($entityType = MediaType::load($entity->bundle()))
              ? $entityType->label()
            : '';
            break;

          default:
            $type = t('Unknown');
        }

        $edit = [
          '#type' => 'submit',
          '#value' => $this->t('Edit'),
          '#ajax' => [
            'url' => Url::fromRoute(
              'entity_browser.edit_form', [
                'entity_type' => $entity->getEntityTypeId(),
                'entity' => $entity->id(),
              ]
            ),
            'options' => [
              'query' => [
                'details_id' => $details_id,
              ],
            ],
          ],
          '#attributes' => [
            'class' => ['edit-button'],
          ],
          '#access' => $edit_button_access,
        ];
        $replace = [
          '#type'                    => 'submit',
          '#value'                   => $this->t('Replace'),
          '#ajax'                    => [
            'callback' => [get_class($this), 'updateWidgetCallback'],
            'wrapper'  => $details_id,
          ],
          '#submit'                  => [
            [
              get_class($this),
              'removeItemSubmit',
            ],
          ],
          '#name'                    => $this->fieldDefinition->getName() . '_replace_' . $entity->id() . '_' . $row_id . '_' . md5(json_encode($field_parents)),
          '#limit_validation_errors' => [array_merge($field_parents, [$this->fieldDefinition->getName()])],
          '#attributes'              => [
            'data-entity-id' => $entity->getEntityTypeId() . ':' . $entity->id(),
            'data-row-id'    => $row_id,
            'class'          => ['replace-button'],
          ],
          '#access'                  => $replace_button_access,
        ];

        $remove = [
          '#type'                    => 'submit',
          '#value'                   => $this->t('Remove'),
          '#ajax'                    => [
            'callback' => [get_class($this), 'updateWidgetCallback'],
            'wrapper'  => $details_id,
          ],
          '#submit'                  => [
            [
              get_class($this),
              'removeItemSubmit',
            ],
          ],
          '#name'                    => $this->fieldDefinition->getName() . '_remove_' . $entity->id() . '_' . $row_id . '_' . md5(json_encode($field_parents)),
          '#limit_validation_errors' => [array_merge($field_parents, [$this->fieldDefinition->getName()])],
          '#attributes'              => [
            'data-entity-id' => $entity->getEntityTypeId() . ':' . $entity->id(),
            'data-row-id'    => $row_id,
            'class'          => ['remove-button'],
          ],
          '#access'                  => (bool) $this->getSetting('field_widget_remove'),
        ];


        $table[] = [
          'title'   => $display,
          'type'    => ['#markup' => $type],
          'actions' => [$edit, $replace, $remove],
        ];
      }

      return $table;
    }
    catch (PluginException $exception) {
      \Drupal::logger('Entity Browser - Table Display')
        ->error(t(
          'Could not get the field widget display: @message',
          ['@message' => $exception->getMessage()]
        ));

      return $table = [
        '#type'       => 'table',
        '#header'     => ['Title', 'Brands', 'Type', 'Actions'],
        '#attributes' => ['class' => $classes],
        '#rows'       => [
          [
            'The field widget could not be found. See the logs for details',
            '',
            '',
            '',
          ],
        ],
      ];
    }

  }

}

